import numpy as np
import scipy.stats as ss
import random


#% Adaptive MCMC, using Haario et al:
#% https://link.springer.com/article/10.1007/s11222-008-9110-y
#% and
#% http://probability.ca/jeff/ftpdir/adaptex.pdf
## *********************************

#% F:          Function giving log-posterior density for a parameter set x
#% x0:         Initial value of parameter set x
#% n:          Number of iterations
#% cov0:       Initial covariance matrix
#% fac:        Scaling factor for covariance matrix. Set fac = 1 for default
#% fixinds:    Elements of x that should be held fixed. Set fixinds = [] for full MCMC
#% blockinds:  Number of 'epi parameters' (e.g. beta, X2) in x, if we want to vary epi and non-epi parameters as independent 'blocks'. Set blockinds = [] if we want a full covariance matrix
#% displ:      Structure with display options. Set displ = true to show progress

def MCMC_adaptive(F, x0, n, sigma, fixinds, blockind, cov0, displ):
    xsto=None; outsto=None; history=None; accept_rate=None
    d = len(x0)
    b = 0.05
    sd = sigma*2.4*2.4/d
    if len(fixinds)!=0:
        inds=fixinds[0,:]
        vals = fixinds[1,:]
    else:
        inds=[]
        vals=[]

    #% Checks on the initial covariance matrix
    if len(cov0) ==0:
        cov0 = np.eye(d)
        cov0[inds,:] = 0
        cov0[:,inds] = 0
        if blockind!=[]:
            cov0[0:blockind,blockind:d] = 0
            cov0[blockind:d,0:blockind] = 0
    #end

    #% Initiate the output matrices
    xsto = np.zeros((d,n)); outsto = np.zeros(n)#xsto = zeros(d,n); outsto = zeros(1,n);
    history = np.zeros((d+1,n))#history = zeros(d+1,n);                                                    % Rows: 1:d Proposed values 4. Accept or reject

    xsto[:,0] = x0[:]; xbar = xsto#xsto(:,1) = x0(:); xbar = xsto;
    FX = F(x0); outsto[1] = FX#FX = F(x0); outsto(1) = FX;
    acc = 0

    current_mean = x0
    current_cov = cov0

    #if displ; figure; end
    # % --- Start the MCMC loop -------------------------------------------------
    for t in range(1,n-1):
        X = xsto[:,t-1]
        #% --- Make a proposal from the distribution
        Y0 =  ss.multivariate_normal(mean=X, cov=0.1*0.1*cov0*sigma/d).rvs()#Y0 = mvnrnd(X,0.1^2*cov0*sigma/d) #??
        if t < 2*d:
           Y = np.maximum(Y0,0); Y[inds] = vals#Y = max(Y0,0); Y(inds) = vals;
        else:
           ind0 = 0; ind1 = t-1
           covmat = np.cov(xsto[:,ind0:ind1].transpose()) #covmat = cov(xsto(:,ind0:ind1)');
                        
           covmat[inds,:] = 0; covmat[:,inds] = 0 #covmat(inds,:) = 0; covmat(:,inds) = 0
           if blockind!=[]:
            covmat[0:(blockind-1),blockind:] = 0; covmat[blockind:,0:(blockind-1)] = 0#covmat(1:blockind,(blockind+1:end)) = 0; covmat((blockind+1):end,1:blockind) = 0;
           #% Need to modify this bit so it goes recursively - faster
        
           covmat = (covmat+covmat.transpose())/2+1e-5#covmat = (covmat+covmat')/2+1e-5;
           #% Precaution to make sure values don't get negative
           Y = np.maximum((1-b)*ss.multivariate_normal(mean=X,cov=sd*covmat) + b*Y0,0) #Y = max((1-b)*mvnrnd(X,sd*covmat) + b*Y0,0);
           Y[inds] = vals#Y(inds) = vals;
        #end

        history[0:d,t] = Y#history(1:d,t) = Y;
        #% --- Decide whether to accept or not
        FY = F(Y)
        if np.random.uniform() < np.exp(FY-FX):
           #% Accept
           xsel = Y[:]#xsel = Y(:);
           FX = FY#;
           acc = acc+1
           history[d,t] = 1#history(end,t) = 1;
        else:
           #% Reject
           xsel = xsto[:,t-1]#xsel = xsto(:,t-1);
        #end
        xsto[:,t] = xsel#;xsto(:,t) = xsel;
        outsto[t] = FX#;outsto(t) = FX;
        xbar[:,t] = (xbar[:,t-1]*(t-1) + xsel)/t#xbar(:,t) = (xbar(:,t-1)*(t-1) + xsel)/t;
        #% Display options
        #if displ && (mod(t,round(n/25))==0); fprintf('%0.5g ', t/n*25); end
        #if displ && (mod(t,200)==0)
        #     plot(xsto(:,1:t-1)'); xlim([0 n]); drawnow;
        #end
    accept_rate = acc/n#;
    xsto = xsto.transpose()#xsto = xsto';
    history = history.transpose()#history = history';    

    return (xsto,outsto,history,accept_rate)


def MCMC_adaptive2(F, x0, n, sigma, fixinds, blockind, cov0, displ):
    xsto=None; outsto=None; history=None; accept_rate=None
    d = len(x0)
    b = 0.05
    sd = sigma*2.4*2.4/d
    if len(fixinds)!=0:
        inds=fixinds[0,:]
        vals = fixinds[1,:]
    else:
        inds=[]
        vals=[]

    #% Checks on the initial covariance matrix
    if len(cov0) ==0:
        cov0 = np.eye(d)
        cov0[inds,:] = 0
        cov0[:,inds] = 0
        if blockind!=[]:
            cov0[0:blockind,blockind:d] = 0
            cov0[blockind:d,0:blockind] = 0
    #end

    #% Initiate the output matrices
    xsto = np.zeros(d,n); outsto = np.zeros(1,n)#xsto = zeros(d,n); outsto = zeros(1,n);
    history = np.zeros(d+1,n)#history = zeros(d+1,n);                                                    % Rows: 1:d Proposed values 4. Accept or reject

    xsto[:,0] = x0[:] #xbar = xsto#xsto(:,1) = x0(:); xbar = xsto;
    FX = F(x0); outsto[1] = FX#FX = F(x0); outsto(1) = FX;
    acc = 0

    current_mean = x0
    current_cov = cov0
    #if displ; figure; end
    # % --- Start the MCMC loop -------------------------------------------------
    for t in range(1,n-1):
        X = xsto[:,t-1]
        #% --- Make a proposal from the distribution
        Y0 =  ss.multivariate_normal(mean=X, cov=0.1*0.1*cov0*sigma/d).rvs()#Y0 = mvnrnd(X,0.1^2*cov0*sigma/d) #??
        if t < 2*d:
           Y = np.maximum(Y0,0); Y[inds] = vals#Y = max(Y0,0); Y(inds) = vals;
        else:
           ind0 = 0; ind1 = t-1
           covmat = np.cov(xsto[:,ind0:ind1].transpose()) #covmat = cov(xsto(:,ind0:ind1)');
                        
           covmat[inds,:] = 0; covmat[:,inds] = 0 #covmat(inds,:) = 0; covmat(:,inds) = 0
           if blockind!=[]:
            covmat[0:(blockind-1),blockind:d] = 0; covmat[blockind:d,0:(blockind-1)] = 0#covmat(1:blockind,(blockind+1:end)) = 0; covmat((blockind+1):end,1:blockind) = 0;
           #% Need to modify this bit so it goes recursively - faster
        
           covmat = (covmat+covmat.transpose())/2+1e-5#covmat = (covmat+covmat')/2+1e-5;
           #% Precaution to make sure values don't get negative
           Y = np.maximum((1-b)*ss.multivariate_normal(mean=X,cov=sd*covmat) + b*Y0,0) #Y = max((1-b)*mvnrnd(X,sd*covmat) + b*Y0,0);
           Y[inds] = vals#Y(inds) = vals;
        #end

        history[0:d,t] = Y#history(1:d,t) = Y;
        #% --- Decide whether to accept or not
        FY = F(Y)
        if random.uniform() < np.exp(FY-FX):
           #% Accept
           xsel = Y[:]#xsel = Y(:);
           FX = FY#;
           acc = acc+1
           history[d,t] = 1#history(end,t) = 1;
        else:
           #% Reject
           xsel = xsto[:,t-1]#xsel = xsto(:,t-1);
        #end
        xsto[:,t] = xsel#;xsto(:,t) = xsel;
        outsto[t] = FX#;outsto(t) = FX;
        #xbar[:,t] = (xbar[:,t-1]*(t-1) + xsel)/t;#xbar(:,t) = (xbar(:,t-1)*(t-1) + xsel)/t;
        #delta_bf = xsel - current_mean
        #current_mean = (current_mean*(t-1)+xsel)/t
        #delta_af = xsel-current_mean
        #current_cov = current_cov + np.outer(delta_bf,delta_af)
        w0 = 1/(t+1); w1 = t/(t+1); w2= w1/(t+1)
        current_mean = w1*current_mean + w0*xsel
        delta_af = xsel-current_mean
        current_cov = w1*current_cov + w2*np.outer(delta_af,delta_af)
        #% Display options
        #if displ && (mod(t,round(n/25))==0); fprintf('%0.5g ', t/n*25); end
        #if displ && (mod(t,200)==0)
        #     plot(xsto(:,1:t-1)'); xlim([0 n]); drawnow;
        #end
    accept_rate = acc/n#;
    xsto = xsto.transpose()#xsto = xsto';
    history = history.transpose()#history = history';    

    return (xsto,outsto,history,accept_rate)


from SpectrumCommon.Modvars.AM.AMModvarShapes import AM_modvar_shapes
from SpectrumCommon.Modvars.GB.GBDefs import gb_update_years
from SpectrumCommon.Const.AM import *

def AM_UpdateYears(projection, new_projection, params):

    projection[AM_InitialConditionsTag] = new_projection[AM_InitialConditionsTag]
    
    gb_update_years(projection, params, AM_modvar_shapes)
    pass



# DP_RetroYearsCalibrationTag         = '<DP_RetroYearsCalibration_V1>'
DP_InitialConditionsTag             = '<DP_InitialConditions_V3>'
# DP_SurvRate1970Tag                  = '<DP_SurvRate1970_V1>'
# DP_FirstYearTag                     = '<DP_FirstYear_V1>'
DP_CustomFileYearsTag               = '<DP_CustomFileYears_V1>'
DP_CustomPopStopRescalingYearTag    = '<DP_CustomPopStopRescalingYear_V1>'
# DP_FinalYearTag                     = '<DP_FinalYear_V1>'
# DP_ProjectionValidTag               = '<DP_ProjectionValid_V1>'
# DP_VersionNumTag                    = '<DP_VersionNum_V1>'
# DP_FileTitleTag                     = '<DP_FileTitle_V1>'
# DP_ValidVersTag                     = '<DP_ValidVers_V1>'
# DP_ValidDateTag                     = '<DP_ValidDate_V1>'
# DP_EdAdSourceTag                    = '<DP_EdAdSource_V1>'
# DP_EdDmSourceTag                    = '<DP_EdDmSource_V1>'
DP_DPSourcesTag                     = '<DP_DPSources_V1>'
DP_DataSourceTag                    = '<DP_DataSource_V1>'
# DP_SourcesTag                       = '<DP_Sources_V1>'
# DP_DefaultDataSourceTag             = '<DP_DefaultDataSource_V1>'
DP_DefaultUPDLETag                  = '<DP_DefaultUPDLE_V1>'
DP_DefaultUPDIMRTag                 = '<DP_DefaultUPDIMR_V1>'
DP_DefaultUPDCMRTag                 = '<DP_DefaultUPDCMR_V1>'
DP_DefaultUPDSurvRateTag            = '<DP_DefaultUPDSurvRate_V1>'
DP_BigPopTag                        = '<DP_BigPop_V1>'
DP_FirstYearPop_Meta_Tag            = '<DP_FirstYearPop_Meta_V1>'
DP_TFRTag                           = '<DP_TFR_V1>'
DP_ASFRTag                          = '<DP_ASFR_V1>'
DP_UNPopASFRTag                     = '<DP_UNPopASFR_V1>'
DP_ASFRTablesTag                    = '<DP_ASFRTables_V1>'
DP_ASFRNumTag                       = '<DP_ASFRNum_V1>'
DP_ASFRCustomFlagTag                = '<DP_ASFRCustomFlag_V1>'
DP_SexBirthRatioTag                 = '<DP_SexBirthRatio_V1>'
DP_LETag                            = '<DP_LE_V1>'
DP_MigrRateTag                      = '<DP_MigrRate_V1>'
DP_MigrAgeDistTag                   = '<DP_MigrAgeDist_V1>'
# DP_COVID19DeathRateTag              = '<DP_COVID19DeathRate_V1>'
# DP_COVID19DeathAgeDistTag           = '<DP_COVID19DeathAgeDist_V1>'
# DP_FirstYearChangedPercentTag       = '<DP_FirstYearChangedPercent_V1>'
DP_ModelLifeFileNameTag             = '<DP_ModelLifeFileName_V1>'
# DP_ScalerTag                        = '<DP_Scaler_V1>'
DP_UseFYrSingleAgesTag              = '<DP_UseFYrSingleAges_V1>'
# DP_EnterCOVID19DeathsTag            = '<DP_EnterCOVID19Deaths_V1>'
DP_RegionalAdjustPopDataTag         = '<DP_RegionalAdjustPopData_V1>'
DP_RegionalAdjustPopCBStateTag      = '<DP_RegionalAdjustPopCBState_V1>'
DP_RegionalAdjustPopFNameTag        = '<DP_RegionalAdjustPopFName_V1>'
DP_RegionalAdjustPopFDateTag        = '<DP_RegionalAdjustPopFDate_V1>'
DP_LifeTableNumTag                  = '<DP_LifeTableNum_V1>'
# DP_DefaultEpidemicTag               = '<DP_DefaultEpidemic_V1>'
# DP_FirstYearOfEpidemicTag           = '<DP_FirstYearOfEpidemic_V1>'
# DP_SingleYearProjectionTag          = '<DP_SingleYearProjection_V1>'
DP_SurvRateTag                      = '<DP_SurvRate_V1>'
DP_BirthsTag                        = '<DP_Births_V1>'
# DP_DeathsTag                        = '<DP_Deaths_V1>'
DP_DeathsByAgeTag                   = '<DP_DeathsByAge_V1>'
DP_IMRTag                           = '<DP_IMR_V1>'
DP_CMRTag                           = '<DP_CMR_V1>'
DP_AdjLETag                         = '<DP_AdjLE_V1>'

# DP_ARTByAgeInputTypeTag             = '<DP_ARTByAgeInputType_V1>'
DP_UALowHighRatiosTag               = '<DP_UALowHighRatios_V1>'
DP_AIDSDeathsFYearUATag             = '<DP_AIDSDeathsFYearUA_V1>'
DP_UAHasRanTag                      = '<DP_UAHasRan_V1>'
DP_UAInfoTag                        = '<DP_UAInfo_V1>'

# result tags
DP_ResultAgeSpecificMortalityTag            = '<DP_ResultAgeSpecificMortality_V1>'
DP_ResultDeathsByAgeLastBirthdayTag         = '<DP_ResultDeathsByAgeAtLastBirthday_V1>'
DP_ResultCWRTag                             = '<DP_ResultCWR_V1>'
DP_ResultGRRTag                             = '<DP_ResultGRR_V1>'
DP_ResultMACTag                             = '<DP_ResultMAC_V1>'
DP_ResultNRRTag                             = '<DP_ResultNRR_V1>'
DP_ResultIMRTag                             = '<DP_ResultIMR_V1>'
DP_ResultU5MRTag                            = '<DP_ResultU5MR_V1>'
DP_Result45q15Tag                           = '<DP_Result45q15_V1>'
DP_ResultCBRTag                             = '<DP_ResultCBR_V1>'
DP_ResultGRTag                              = '<DP_ResultGR_V1>'
DP_ResultCDRTag                             = '<DP_ResultCDR_V1>'
DP_ResultDoublingTimeTag                    = '<DP_ResultDoublingTime_V1>'
DP_ResultRNITag                             = '<DP_ResultRNI_V1>'
DP_ResultMedianAgeTag                       = '<DP_ResultMedianAge_V1>'

# temp calculation tags
# DP_BirthsSATempTag          = '<DP_BirthsSA_V1>'
# DP_DeathsSATempTag          = '<DP_DeathsSA_V1>'
# DP_Pop1TempTag              = '<DP_pop1_V1>'
# DP_SAMigrRateTempTag        = '<DP_SingleAgeMigrRate_V1>'
# DP_VMatrixTempTag           = '<DP_vMatrix_V1>'
# DP_SexRatioAtBirthTempTag   = '<DP_Sex_ratio_at_birth_V1>'
# DP_SurvRateTempTag          = '<DP_SurvRateTemp_V1>'
DP_Pop1TempTransportTag                     = '<pop1TempTransport>'
DP_InitialConditionsTempTransportTag        = '<InitialConditionsTempTransport>'



# Input ModVars

IC_VERSION_TAG_V1                                  = '<IC_Version_V1>'
IC_DATA_CHANGED_TAG_V1                             = '<IC_DataChanged_V1>'
IC_PROJ_VALID_TAG_V1                               = '<IC_ProjectionValid_V1>'
IC_PROJ_VALID_DATE_TAG_V1                          = '<IC_ProjectionValidDate_V1>'
IC_SHOW_ALL_YEARS_INTERV_COV_TAG_V1                = '<IC_ShowAllYearsIntervCov_V1>'
IC_SHOW_ALL_YEARS_INTERV_PIN_TAG_V1                = '<IC_ShowAllYearsIntervPIN_V1>'
IC_SHOW_ALL_FP_METHODS_INTERV_COV_TAG_V1           = '<IC_ShowAllFPMethodsIntervCov_V1>'
IC_ENTER_DET_PMTCT_DATA_TAG_V1                     = '<IC_EnterDetailedPMTCT_Data_V1>'
IC_ENTER_NET_AGE_DIST_PERC_TAG_V1                  = '<IC_EnterNetAgeDistribAsPerc_V1>'
IC_POST_TREAT_SURV_FOLLOW_UP_VIS_TAG_V1            = '<IC_PostTreatSurvFollowUpVisited_V1>'
IC_MAX_CUSTOM_DRUG_SUPP_MST_ID_TAG_V1              = '<IC_MaxCustomDrugSupplyMstID_V1>'
IC_COV_YEAR_BEF_BASE_YEAR_ITN_TAG_V1               = '<IC_CovYearBeforeBaseYearITN_V1>'
IC_NET_LIFETIME_ITN_TAG_V1                         = '<IC_NetLifetimeITN_V1>'
IC_AVG_NET_AGE_ITN_TAG_V1                          = '<IC_AvgNetAgeITN_V1>'
IC_NUM_PERC_NETS_BY_AGE_TAG_V1                     = '<IC_NumPercNetsByAge_V1>'
IC_DRUG_PRICE_ADJUST_TAG_V1                        = '<IC_DrugPriceAdjust_V1>'
IC_INTERV_RECORDS_TAG                              = '<IC_InterventionRecords_V4>' 
IC_TREAT_INPUT_DRUG_SUPPLY_RECORDS_TAG             = '<IC_TreatInputDrugSupplyRecords_V2>' # Treatment inputs = Intervention inputs     
IC_TREAT_INPUT_MED_PERSONNEL_RECORDS_TAG           = '<IC_TreatInputMedPersonnelRecords_V2>' # Medical personnel = Health providers      
IC_TREAT_INPUT_OUTPAT_INPAT_RECORDS_TAG            = '<IC_TreatInputOutpatInpatRecords_V2>'        
IC_TREAT_INPUT_GROUP_DRUG_SUPPLY_RECORDS_TAG       = '<IC_TreatInputGroupDrugSupplyRecords_V2>'     
IC_TREAT_INPUT_GROUP_MED_PERSONNEL_RECORDS_TAG     = '<IC_TreatInputGroupMedPersonnelRecords_V2>'   
IC_TREAT_INPUT_GROUP_OUTPAT_INPAT_RECORDS_TAG      = '<IC_TreatInputGroupOutpatInpatRecords_V2>'   
IC_INTERV_STRATUM_RECORDS_TAG_V1                   = '<IC_InterventionStratumRecords_V1>'
IC_DRUG_SUPPLY_RECORDS_TAG                         = '<IC_DrugSupplyRecords_V3>' 
IC_TARG_POP_RECORDS_TAG                            = '<IC_TargPopRecords_V1>'
IC_NEED_BRST_CNCR_TRT_FLLW_UP_TAG_V1               = '<IC_NeedBrstCncrTrtFllwUp_V1>'
IC_OWNER_USAGE_RATIO_ARRAY_ITN_TAG_V1              = '<IC_OwnerUsageRatioArrayITN_V1>'
IC_STRATA_INTERV_LIST_TAG_V1                       = '<IC_StrataIntervList_V1>'
IC_ALT_COV_LIST_TAG_V1                             = '<IC_AltCoverageList_V1>'
IC_ALT_PIN_AGE_GROUP_SET_1_LIST_TAG_V1             = '<IC_AltPINAgeGroupSet1List_V1>'
IC_ALT_PIN_AGE_GROUP_SET_2_LIST_TAG_V1             = '<IC_AltPINAgeGroupSet2List_V1>'    
IC_STRATUM_RECORDS_TAG_V1                          = '<IC_StratumRecords_V1>'
IC_OUTP_VISIT_INP_DAY_UNIT_COST_TAG_V1             = '<IC_OutpVisitInpDayUnitCost_V1>'
IC_NON_DEF_TARG_POPS_TAG_V1                        = '<IC_NonDefaultTargPops_V1>'
IC_VARY_TREATMENT_INPUTS_BY_TIME_TAG_V1            = '<IC_VaryTreatmentInputsByTime_V1>'
# by intervention in intervention records now IC_POP_IN_NEED_VISITED_TAG_V1                      = '<IC_PopInNeedVisited_V1>'
IC_FAM_PLAN_GOAL_TAG_V1                            = '<IC_FamPlanGoal_V1>'
IC_PERC_UNMET_NEED_COVERED_TAG_V1                  = '<IC_PercUnmetNeedCovered_V1>'

# Inputs - TB Costing

IC_INTERV_RECORDS_IHT_ID_INFO_ONLY_TB_TAG_V1 = '<IC_InterventionRecordsIHT_IDInfoOnly_V1>'

# Inputs - LiST Costing
                                                                                                                           
IC_OTHER_RECURRENT_COSTS_CS_TAG_V1                     = '<IC_OtherRecurrentCostsCS_V1>'
IC_CAPITAL_COSTS_CS_TAG_V1                             = '<IC_CapitalCostsCS_V1>'
IC_SUPPLY_CHAIN_COST_PERC_COMMODITY_COST_CS_TAG_V1     = '<IC_SupplyChainCostPercCommodityCostCS_V1>' # = 'Logistics'
IC_WASTAGE_CS_TAG_V1                                   = '<IC_WastageCS_V1>'
IC_INFRASTRUCTURE_INVESTMENT_CS_TAG_V1                 = '<IC_InfrastructureInvestmentCS_V1>' 
IC_INFRASTRUCTURE_ALLOC_PORTION_FOR_PROJ_CS_TAG_V1     = '<IC_InfrastructureAllocPortionForProjCS_V1>'
IC_RATIO_OTHER_HEALTH_SYSTEM_TO_INTERV_COSTS_CS_TAG_V1 = '<IC_RatioOtherHealthSystemToIntervCostsCS_V1>'
IC_HEALTH_SYSTEM_PROG_COSTS_SUM_CS_TAG_V1              = '<IC_HealthSystemProgrammeCostsSumCS_V1>'
IC_OTHER_HEALTH_SYSTEM_COSTS_VIS_CS_TAG_V1             = '<IC_OtherHealthSystemCostsVisitedCS_V1>'

IC_INCOME_LEVEL_CS_META_TAG_V1                         = '<IC_IncomeLevelCS_Meta_V1>'
IC_SUPPLY_CHAIN_STATUS_CS_META_TAG_V1                  = '<IC_SupplyChainStatusCS_Meta_V1>'

# Inputs - Comprehensive vs Specific (CVS) plan mode

IC_OUTPAT_INPAT_COSTS_CVS_TAG_V1                    = '<IC_OutpatInpatCostsCVS_V1>'

# Pre-result / Calculated ModVars

IC_DRUG_COST_PER_CASE_TAG_V1                       = '<IC_DrugCostPerCase_V1>'
IC_SUPPLY_COST_PER_CASE_TAG_V1                     = '<IC_SupplyCostPerCase_V1>'
IC_LABOR_COST_PER_CASE_TAG_V1                      = '<IC_LaborCostPerCase_V1>'
IC_MIN_PER_AVG_CASE_STATIC_STAFF_TAG_V1            = '<IC_MinPerAvgCaseDefaultStaff_V1>'
IC_MIN_PER_AVG_CASE_DYNAMIC_STAFF_TAG_V1           = '<IC_MinPerAvgCaseCustomStaff_V1>'
IC_INPATIENT_DAYS_PER_CASE_TAG_V1                  = '<IC_InpatientDaysPerCase_V1>'
IC_OUTPATIENT_VISITS_PER_CASE_TAG_V1               = '<IC_OutpatientVisitsPerCase_V1>'

# Output / Result ModVars

IC_RES_TARG_POPS_TAG_V1                                        = '<IC_ResTargetPopulations_V1>'
IC_RES_PEOPLE_NEEDING_SERVICES_TAG_V1                          = '<IC_ResPeopleNeedingServices_V1>'
IC_RES_NUM_SERVICES_TAG_V1                                     = '<IC_ResNumServices_V1>'
IC_RES_NUM_SERVICES_BY_DELIV_CHAN_TAG_V1                       = '<IC_ResNumServicesByDelivChan_V1>'
IC_RES_ADDIT_NUM_SERVICES_BY_DELIV_CHAN_TAG_V1                 = '<IC_ResAdditNumServicesByDelivChan_V1>'
IC_RES_ADDIT_NUM_SERVICES_TAG_V1                               = '<IC_ResAdditNumServices_V1>'
IC_RES_PERC_DELIV_THROUGH_CHAN_BY_YEAR_TAG_V1                  = '<IC_ResPercDeliveredThroughChanByYear_V1>'
IC_RES_DRUG_COSTS_TAG_V1                                       = '<IC_ResDrugCosts_V1>'
IC_RES_ADDIT_DRUG_COSTS_TAG_V1                                 = '<IC_ResAdditDrugCosts_V1>'
IC_RES_DRUG_COSTS_BY_YEAR_ONLY_TAG_V1                          = '<IC_ResDrugCostsByYearOnly_V1>'
IC_RES_SUPPLY_COSTS_TAG_V1                                     = '<IC_ResSupplyCosts_V1>'
IC_RES_ADDIT_SUPPLY_COSTS_TAG_V1                               = '<IC_ResAdditSupplyCosts_V1>'
IC_RES_SUPPLY_COSTS_BY_YEAR_ONLY_TAG_V1                        = '<IC_ResSupplyCostsByYearOnly_V1>'
IC_RES_ADDIT_DRUG_SUPPLY_COSTS_TAG_V1                          = '<IC_ResAdditDrugSupplyCosts_V1>'
IC_RES_STATIC_STAFF_TIME_REQ_BY_DELIV_CHAN_TAG_V1              = '<IC_ResStaffTimeReqByDelivChan_V1>'
IC_RES_STATIC_STAFF_TIME_REQ_BY_INTERV_TAG_V1                  = '<IC_ResStaffTimeReqByInterv_V1>'
IC_RES_STATIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1               = '<IC_ResStaffTimeReqByProgArea_V1>'
IC_RES_STATIC_STAFF_TIME_REQ_BY_PROG_AREA_DELIV_CHAN_TAG_V1    = '<IC_ResStaffTimeReqByProgAreaDelivChan_V1>'
IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_DELIV_CHAN_TAG_V1             = '<IC_ResCustomStaffTimeReqByDelivChan_V1>'
IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_INTERV_TAG_V1                 = '<IC_ResCustomStaffTimeReqByInterv_V1>' 
IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1              = '<IC_ResCustomStaffTimeReqByProgArea_V1>'
IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_PROG_AREA_DELIV_CHAN_TAG_V1   = '<IC_ResCustomStaffTimeReqByProgAreaDelivChan_V1>'
IC_RES_DRUG_UNITS_PER_AVG_CASE_TAG_V1               = '<IC_ResDrugUnitsPerAvgCase_V1>'
IC_RES_SUPPLY_UNITS_PER_AVG_CASE_TAG_V1             = '<IC_ResSupplyUnitsPerAvgCase_V1>'
IC_RES_DRUGS_REQUIRED_BY_DELIV_CHAN_TAG             = '<IC_ResDrugsRequired_V1>'
IC_RES_SUPPLIES_REQUIRED_BY_DELIV_CHAN_TAG          = '<IC_ResSuppliesRequired_V1>'
IC_RES_DRUGS_REQUIRED_BY_PROG_AREA_TAG              = '<IC_ResDrugsRequiredByProgArea_V1>'
IC_RES_SUPPLIES_REQUIRED_BY_PROG_AREA_TAG           = '<IC_ResSuppliesRequiredByProgArea_V1>'
IC_RES_OUTPAT_VISITS_INPAT_DAYS_TAG_V1              = '<IC_ResOutpatVisitsInpatDays_V1>'

# Outputs - Disease-specific costing (DSC), TB Costing, LiST Costing

IC_RES_LABOR_COSTS_DSC_TAG_V1                       = '<IC_ResLaborCostsDSC_V1>'

# Outputs - Disease-specific costing (DSC), TB Costing

#IC_OUTPAT_VISIT_INPAT_DAY_UNIT_COSTS_DSC_TAG_V1     = '<IC_OutpatVisitInpatDayUnitCostsDSC_V1>'
IC_RES_OUTPAT_VISIT_INPAT_DAY_COSTS_DSC_TAG_V1   = '<IC_ResOutpatVisitInpatDayCostsDSC_V1>'
IC_RES_LOG_SUPPLY_CHAIN_COSTS_TB_TAG             = '<IC_ResLogisticsSupplyChainCostsTB_V1>'
IC_RES_WASTAGE_COSTS_TB_TAG                      = '<IC_ResWastageCostsTB_V1>'

# Outputs - LiST Costing

IC_RES_OTHER_RECURRENT_COSTS_CS_TAG_V1           = '<IC_ResOtherRecurrentCostsCS_V1>'
IC_RES_CAPITAL_COSTS_CS_TAG_V1                   = '<IC_ResCapitalCostsCS_V1>'
IC_RES_PROGRAM_COSTS_CS_TAG_V1                   = '<IC_ResProgramCostsCS_V1>'
IC_RES_LOGISTICS_COSTS_CS_TAG_V1                 = '<IC_ResLogisticsCostsCS_V1>'
IC_RES_WASTAGE_COSTS_CS_TAG_V1                   = '<IC_ResWastageCostsCS_V1>'
IC_RES_INFRASTRUCTURE_INVESTMENT_COSTS_CS_TAG_V1 = '<IC_ResInfrastructureInvestmentCostsCS_V1>'
IC_RES_OTHER_HEALTH_SYSTEM_COSTS_CS_TAG_V1       = '<IC_ResOtherHealthSystemCostsCS_V1>'

IC_COSTS_BY_INT_CS_TAG_V1                        = '<IC_CostsByInt_V1>'
IC_DISC_SOCIAL_BENEFITS_BY_INT_CS_TAG_V1         = '<IC_DiscountedSocialBenefitsByInt_V1>'
IC_DISC_ECON_BENEFITS_BY_INT_CS_TAG_V1           = '<IC_DiscountedEconomicBenefitsByInt_V1>'
IC_DISC_INC_IN_EARN_REDUC_BY_INT_CS_TAG_V1       = '<IC_DiscIncInLifetimeEarnReducByInt_V1>'
IC_BCR_BY_INT_CS_TAG_V1                          = '<IC_BCRByInt_V1>'


from SpectrumCommon.Const.TB import *
from SpectrumCommon.Modvars.TB.TBScreeningDefs import *

tb_patient_indices = {
    
    # Pulmonary TB
    'Pulmonary TB HIV- 0-14'  : TB_PTB_HIVn_014_grp ,
    'Pulmonary TB HIV- 15+'   : TB_PTB_HIVn_15p_grp ,
    'Pulmonary TB HIV+ 0-9'   : TB_PTB_HIVp_09_grp  ,
    'Pulmonary TB HIV+ 10-14' : TB_PTB_HIVp_1014_grp,
    'Pulmonary TB HIV+ 15+ ' : TB_PTB_HIVp_15p_grp ,

    # Extra-Pulmonary TB
    'Extra-pulmonary TB HIV- 0-14'   : TB_ETB_HIVn_014_grp ,
    'Extra-pulmonary TB HIV- 15+'    : TB_ETB_HIVn_15p_grp ,
    'Extra-pulmonary TB HIV+ 0-9'    : TB_ETB_HIVp_09_grp  ,
    'Extra-pulmonary TB HIV+ 10-14'  : TB_ETB_HIVp_1014_grp,
    'Extra-pulmonary TB HIV+ 15+ '  : TB_ETB_HIVp_15p_grp 
}

tb_screening_indices = {
    'Screened'                     : TB_ScreenedNum,
    'Screened number referred'     : TB_ScreenedNumReferred,
    'Screened true postives (TP)'  : TB_ScreenedTP,
    'Screened false postives (FP)' : TB_ScreenedFP,
    'Screened true cases'          : TB_ScreenedTrueCases,
    'Screened NNTS'                : TB_ScreenedNNTS,
    'Screened TP and FP'           : TB_ScreenedTPFP,
    'Screened TP missed'           : TB_ScreenedTPMissed
}

tb_diagnosis_indices = {
    'Diagnosis'                      : TB_DiagnosisNum, 
    'Diagnosis Confirmed'            : TB_DiagnosisConfirmed,
    'Diagnosis true postives (TP)'   : TB_DiagnosisTP,
    'Diagnosis false positives (FP)' : TB_DiagnosisFP,
    'Diagnosis true cases'           : TB_DiagnosisTrueCases,
    'Diagnosis NNTT'                 : TB_DiagnosisNNTT,
    'Diagnosis TP and FP'            : TB_DiagnosisTPFP,
    'Diagnosis TP Missed '          : TB_DiagnosisTPMissed ,
}

tb_patient_targ_pop_indices = {
    'Clinically assessed'   : TB_ClinicallyAssessed,
    'Referred Dx'           : TB_ReferredDx, 
    'Diagnosis TB'          : TB_DiagnosisTB, 
    'Treatment initialized' : TB_TreatmentInit
}

tb_provider_targ_pop_indices = {
    'Number eligible for screening'   : TB_NumEligibleScreen,
    'Number screened'                 : TB_NumScreened,
    'Number referredDiag'             : TB_NumReferredDiag,
    'Number diagnosed with PTB'       : TB_NumDiagPulmTB,
    'Number initialized on Treatment' : TB_NumInitTreatment,
    'Number eligible for evaluation'  : TB_NumEligibleEval,
    'Number tested for infection'     : TB_NumTestedInfect,
    'Number confirmed infected'       : TB_NumConfirmInfect,
    'Number initialized on TPT'       : TB_NumInitTPT,
}

tb_contact_indices = {
    'Total contacts'          : TB_TotalContacts,
    'Disease infect contacts' : TB_DiseaseInfectCntct,
    'Infection contacts'      : TB_InfectionCntct, 
    'Disease contacts'        : TB_DiseaseCntct,
    'Pulmonary TB contacts'   : TB_PulmTBCntct
}

tb_household_indices = {
    'Household 0 to 4'  : TB_HH0_4, 
    'Household 5 to 14' : TB_HH5_14,
    'Household 15+'     : TB_HH15p 
}

tb_art_age_indices ={
    'ART 0 to 9'   : TB_ART0_9,   
    'ART 10 to 14' : TB_ART10_14,
    'ART 15+'      : TB_ART15p  
}

tb_art_severity_indices = {
    'Not servere'    : TB_ARTNotServere,
    'Severe illness' : TB_ARTSevereIllness
}

tb_hrg_indices ={
    'Prisoners'                                                                         : TB_HRG_Prisoners,
    'Miners exposed to silica '                                                        : TB_HRG_Miners,
    'People with risk factors for TB seeking health care'                               : TB_HRG_SeekingCare,
    'Populations with structural risk factors for TB and limited access to health care' : TB_HRG_StructuralRFs,
    'General population in settings with high TB'                                       : TB_HRG_HighTBPrev,
    'People with untreated fibrotic lesions on chest x-ray'                             : TB_HRG_UntreatedFibrosis,
}

tb_pulmonary_indices = {
    'pulmonary tb'       : TB_PTB,
    'extra-pulmonary tb' : TB_ETB,
}

tb_child_adult_indices = {
     'Child' : TB_Child,
     'Adul' : TB_Adult,
}

tb_cases_indices = {
    'New cases' : TB_NewCases,
    'Retreated cases' : TB_RetreatCases ,
}

tb_resist_age_indices = {
    '0 to 14' : TB_ResistAge0_14,
    '15+' : TB_ResistAge15p ,
}

tb_coverage_indices = {
    'Coverage RR_DST_PTB'          : TB_Cov_RR_DST_PTB,
    'Coverage RR_DST_ETB'          : TB_Cov_RR_DST_ETB,
    'Coverage INH_RR_Sens_DST_PTB' : TB_Cov_INH_RR_Sens_DST_PTB,
    'Coverage INH_RR_Sens_DST_ETB' : TB_Cov_INH_RR_Sens_DST_ETB,
    'Coverage FQ_RR_Res_DST_PTB'   : TB_Cov_FQ_RR_Res_DST_PTB,
    'Coverage FQ_RR_Res_DST_ETB'   : TB_Cov_FQ_RR_Res_DST_ETB,
    'Coverage BDQ RR Res DST PTB'  : TB_Cov_BDQ_RR_Res_DST_PTB ,
    'Coverage BDQ RR Res DST ETB'  : TB_Cov_BDQ_RR_Res_DST_ETB ,
    'Coverage BDQ FQ Res DST PTB'  : TB_Cov_BDQ_FQ_Res_DST_PTB ,
    'Coverage BDQ FQ Res DST ETB'  : TB_Cov_BDQ_FQ_Res_DST_ETB ,
    'Coverage LZD RR Res DST PTB'  : TB_Cov_LZD_RR_Res_DST_PTB ,
    'Coverage LZD RR Res DST ETB'  : TB_Cov_LZD_RR_Res_DST_ETB ,
    'Coverage LZD FQ Res DST PTB'  : TB_Cov_LZD_FQ_Res_DST_PTB ,
    'Coverage LZD FQ Res DST ETB'  : TB_Cov_LZD_FQ_Res_DST_ETB ,
    'Coverage EMC XDR DST XDR'     : TB_Cov_EMC_XDR_DST_XDR,
    'Coverage EMC RR Res DST TB'   : TB_Cov_EMC_RR_Res_DST_TB,
    'Coverage ETO XDR_DST_XDR'     : TB_Cov_ETO_XDR_DST_XDR,
    'Coverage ETO RR_Res_DST_TB'   : TB_Cov_ETO_RR_Res_DST_TB,
    'Coverage PZA XDR_DST_XDR'     : TB_Cov_PZA_XDR_DST_XDR,
    'Coverage PZA RR_Res_DST_TB'   : TB_Cov_PZA_RR_Res_DST_TB,
    'Coverage DR_BDQ_LZD_MOX'      : TB_CovXDR_BDQ_LZD_MOX
}   

tb_impact_indices = {
    'Counterfactual' : TB_Counterfactual, 
    'Scaleup' : TB_Scaleup,
    'Difference' : TB_Difference,
}

tb_result_bound_indices = {
    'Lower' : TB_ResLowerIdx,
    'Mid'   : TB_ResMidIdx,
    'Upper' : TB_ResUpperIdx,
    'SD'    : TB_ResSdIdx 
}

tb_hiv_coinfected_indices = {
   'Number PLHIV with TB '                                                         : TB_HIVCoInf,
   'Number PLHIV with TB, adults'                                                   : TB_HIVCoInfAdult,
   'Number PLHIV with TB, children'                                                 : TB_HIVCoInfChild,
   'Number PLHIV with TB, neonates'                                                 : TB_HIVCoInfNeonates,
   'Number failing this first-line HIV regimen: TDF + 3TC (or FTC) + DTG, adults'   : TB_HIVCoInfFailAdult,
   'Number failing this first-line HIV regimen: TDF + 3TC (or FTC) + DTG, children' : TB_HIVCoInfFailChild,
   'Number failing this first-line HIV regimen: TDF + 3TC (or FTC) + DTG, neonates' : TB_HIVCoInfFailNeonates
}

tb_scenario_indices = {
    'Average probability to detect'             : TB_AvgProbDetect,
    'Average probability to receive prevention' : TB_AvgProbReceivPrvnt,
    'Treatment success'                         : TB_TreatmentSuccess
}

tb_notification_indices = {
    #TB notification case types
    'New cases'                                         : TB_NewCases,
    'New cases, Pulmonary, bacteriologically confirmed' : TB_NewBacterConfirmCases,
    'New cases, Pulmonary, clinically diagnosed cases'  : TB_NewClinicDiagCases,
    'New cases,  extrapulmonary cases'                  : TB_NewExtrapulmonaryCases,

    'Relapse cases'                                         : TB_RelapseCases,
    'Relapse cases, Pulmonary, bacteriologically confirmed' : TB_RelapseBacterConfirmCases,
    'Relapse cases, Pulmonary, clinically diagnosed'        : TB_RelapseClinicDiagCases,
    'Relapse cases, extrapulmonary'                         : TB_RelapseExtrapulmonaryCases,

    'New and relapsed cases'                      : TB_NewAndRelapsedCases,
    'Previously treated, excluding relapse cases' : TB_PrevTreatExcludeRelapseCases,
    'Previously treated, including relapse cases' : TB_PrevTreatIncludeRelapseCases,
    # #Split by age*                    
    'Children 0-9 years'                    : TB_NewRelapse0_9Cases,
    'Children 10-14 years, new and relapse' : TB_NewRelapse10_14Cases,
    'Adults 15+ years'                      : TB_NewRelapse15pCases,

    # #Split by HIV status*             
    'HIV+'                                                        : TB_HIVpCases,
    'HIV+ Adults 15+ years'                                       : TB_HIVp15pCases,
    'HIV+ Children 0-14 years'                                    : TB_HIVp0_14Cases,
    'HIV+ Neonates'                                               : TB_HIVpNeonateCases,
    'Not on ART'                                                  : TB_NotOnARTCases,
    'On ART'                                                      : TB_OnARTCases,
    'On ART Adults 15+ years, failing this frst-line regimen '   : TB_OnART15pCases,
    'On ART Children 0-14 years, failing this first-line regimen' : TB_OnARRT0_14Cases,
    'On ART Neonates, failing this first-line regimen'            : TB_OnARTNeonateCases,
    'Proportion HIVp in pop that excludes ART'                    : TB_HIVpExcludeART
}

tb_resistance_indices = {
    'SEN' : TB_SEN,
    'MDR' : TB_MDR
}


tb_resistance_age_indices = {
    'Age 0-14' : TB_ResistAge0_14,
    'Age 15+'  : TB_ResistAge15p
}

tb_resistance_profile_indices = {
    'Rif Sen' : TB_Rif_Sen,
    'INH Sen' : TB_INH_Sen,
    'INH Res' : TB_INH_Res,
    'Rif Res' : TB_Rif_Res,
    'FQ_Sen'  : TB_FQ_Sen ,
    'FQ_Res'  : TB_FQ_Res ,
    'XDR'     : TB_XDR    
}
TB_ResistProfileLen = 7


tb_num_tested_indices = {
    'PTB RIF INH MDR'      : TB_NumTestPTB_RIF_INH_MDR,
    'PTB INH MDR RIF Sens' : TB_NumTestPTB_INHMDR_RIFSens,
    'PTB FQ RIF MDR'       : TB_NumTestPTB_FQ_RIF_MDR,
    'PTB BDQ RIF MDR'      : TB_NumTestPTB_BDQ_RIF_MDR,
    'PTB BDQ FQ MDR'       : TB_NumTestPTB_BDQ_FQ_MDR,
    'PTB LZD RIF MDR'      : TB_NumTestPTB_LZD_RIF_MDR,
    'PTB LZD FQ MDR'       : TB_NumTestPTB_LZD_FQ_MDR ,
    'ETB RIF INH MDR'      : TB_NumTestETB_RIF_INH_MDR,
    'ETB INH MDR RIF Sens' : TB_NumTestETB_INHMDR_RIFSens,
    'ETB FQ RIF MDR'       : TB_NumTestETB_FQ_RIF_MDR,
    'ETB BDQ RIF MDR'      : TB_NumTestETB_BDQ_RIF_MDR,
    'ETB BDQ FQ MDR'       : TB_NumTestETB_BDQ_FQ_MDR,
    'ETB LZD RIF MDR'      : TB_NumTestETB_LZD_RIF_MDR,
    'ETB LZD FQ MDR'       : TB_NumTestETB_LZD_FQ_MDR,
    'XDR Amikacin'         : TB_NumTextXDR_Amikacin,
    'XDR Ethionamide'      : TB_NumTextXDR_Ethionamide,
    'XDR Pyrazinamide'     : TB_NumTextXDR_Pyrazinamide,
    'XDR BDQ_LZD_MOX'      : TB_NumTextXDR_BDQ_LZD_MOX

}

tb_num_confirmed_indices = {
    'RIF INH Sen' : TB_NumConfRIF_INHSen    ,
    'RIF Sen INH MDR' : TB_NumConfRIFSen_INHMDR ,
    'RIF MDR FQ Sen' : TB_NumConfRIFMDR_FQSen  ,
    'RIF MDR FQ MDR' : TB_NumConfRIFMDR_FQMDR  ,
    'Atleast one' : TB_NumConfAtLeastOne    ,
    'Meningitis Peri' : TB_NumConfMeningitisPeri,
}

tb_case_indices = {
    'New cases' : TB_NewCases,
    'Retreated cases' : TB_RetreatCases
}

tb_art_char_indices = {
    'With TB infection' :  TB_ART_WithTBInfection,
    'With TB disease'   :  TB_ART_WithTBDisease,
    'Percent PTB'       :  TB_ART_PercPTB
}

tb_art_all_severity = {
    'Total' : 0,
    'Without serious illness':1,
    'With serious illness':2
}

tb_screening_options = {
    'g_sym_cgh2w' : TB_Scrn_g_sym_cgh2w,
    'g_sym_any'   : TB_Scrn_g_sym_any,
    'g_mwrd_sn'   : TB_Scrn_g_mwrd_sn,
    'g_sym_cgh2w_cxr_tb_sequential_pos' : TB_Scrn_g_sym_cgh2w_cxr_tb_sequential_pos,
    'g_sym_cgh2w_cxr_tb_sequential_neg' : TB_Scrn_g_sym_cgh2w_cxr_tb_sequential_neg,
    'g_sym_cgh2w_cxr_tb_parallel' : TB_Scrn_g_sym_cgh2w_cxr_tb_parallel,
    'g_sym_cgh2w_cxr_cad_sequential_pos' : TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_pos,
    'g_sym_cgh2w_cxr_cad_sequential_neg' : TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_neg,
    'g_sym_cgh2w_cxr_cad_parallel' : TB_Scrn_g_sym_cgh2w_cxr_cad_parallel,
    'g_sym_any_cxr_tb_sequential_pos' : TB_Scrn_g_sym_any_cxr_tb_sequential_pos,
    'g_sym_any_cxr_tb_sequential_neg' : TB_Scrn_g_sym_any_cxr_tb_sequential_neg,
    'g_sym_any_cxr_tb_parallel' : TB_Scrn_g_sym_any_cxr_tb_parallel,
    'g_sym_any_cxr_cad_sequential_pos' : TB_Scrn_g_sym_any_cxr_cad_sequential_pos,
    'g_sym_any_cxr_cad_sequential_neg' : TB_Scrn_g_sym_any_cxr_cad_sequential_neg,
    'g_sym_any_cxr_cad_parallel' : TB_Scrn_g_sym_any_cxr_cad_parallel,
    'h_clhiv_sym' : TB_Scrn_h_clhiv_sym,
    'h_hiv_mwrd_sn' : TB_Scrn_h_hiv_mwrd_sn,
    'h_hiv_w4ss' : TB_Scrn_h_hiv_w4ss,
    'h_hiv_crp' : TB_Scrn_h_hiv_crp,
    'h_hiv_cxr_abn' : TB_Scrn_h_hiv_cxr_abn,
    'h_hiv_w4ss_hiv_crp_sequential_pos' : TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_pos,
    'h_hiv_w4ss_hiv_crp_sequential_neg' : TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_neg,
    'h_hiv_w4ss_hiv_crp_parallel' : TB_Scrn_h_hiv_w4ss_hiv_crp_parallel,
    'h_hiv_w4ss_hiv_cxr_abn_sequential_pos' : TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_pos,
    'h_hiv_w4ss_hiv_cxr_abn_sequential_neg' : TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_neg,
    'h_hiv_w4ss_hiv_cxr_abn_parallel' : TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_parallel,
    'h_hiv_w4ss_cxr_cad_sequential_pos' : TB_Scrn_h_hiv_w4ss_cxr_cad_sequential_pos,
    'h_hiv_w4ss_cxr_cad_sequential_neg' : TB_Scrn_h_hiv_w4ss_cxr_cad_sequential_neg,
    'h_hiv_w4ss_cxr_cad_parallel' : TB_Scrn_h_hiv_w4ss_cxr_cad_parallel,
    'c_cc_sym' : TB_Scrn_c_cc_sym,
    'c_cc_cxr_tb' : TB_Scrn_c_cc_cxr_tb,
    'c_hiv_w4ss_cc_cxr_tb_sequential_pos' : TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_pos,
    'c_hiv_w4ss_cc_cxr_tb_sequential_neg' : TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_neg,
    'c_hiv_w4ss_cc_cxr_tb_parallel' : TB_Scrn_c_hiv_w4ss_cc_cxr_tb_parallel,
    'c_hiv_w4ss_cxr_cad_sequential_pos' : TB_Scrn_c_hiv_w4ss_cxr_cad_sequential_pos,
    'c_hiv_w4ss_cxr_cad_sequential_neg' : TB_Scrn_c_hiv_w4ss_cxr_cad_sequential_neg,
    'c_hiv_w4ss_cxr_cad_parallel' : TB_Scrn_c_hiv_w4ss_cxr_cad_parallel,
}


tb_patients_opt = TB_screening_alg_patient_options()
tb_patient_screening_options = {
    
    # Pulmonary TB
    'Pulmonary TB HIV- 0-14'  : tb_patients_opt[TB_PTB_HIVn_014_grp],
    'Pulmonary TB HIV- 15+'   : tb_patients_opt[TB_PTB_HIVn_15p_grp],
    'Pulmonary TB HIV+ 0-9'   : tb_patients_opt[TB_PTB_HIVp_09_grp],
    'Pulmonary TB HIV+ 10-14' : tb_patients_opt[TB_PTB_HIVp_1014_grp],
    'Pulmonary TB HIV+ 15+ '  : tb_patients_opt[TB_PTB_HIVp_15p_grp],

    # Extra-Pulmonary TB
    'Extra-pulmonary TB HIV- 0-14'  : tb_patients_opt[TB_ETB_HIVn_014_grp],
    'Extra-pulmonary TB HIV- 15+'   : tb_patients_opt[TB_ETB_HIVn_15p_grp],
    'Extra-pulmonary TB HIV+ 0-9'   : tb_patients_opt[TB_ETB_HIVp_09_grp],
    'Extra-pulmonary TB HIV+ 10-14' : tb_patients_opt[TB_ETB_HIVp_1014_grp],
    'Extra-pulmonary TB HIV+ 15+ '  : tb_patients_opt[TB_ETB_HIVp_15p_grp]
}

tb_household_opt = TB_screening_alg_household_options()
tb_household_screening_options = {
    'Household 0 to 4'  : tb_household_opt[TB_HH0_4], 
    'Household 5 to 14' : tb_household_opt[TB_HH5_14],
    'Household 15+'     : tb_household_opt[TB_HH15p] 
}

tb_art_screen_opt = TB_screening_alg_art_options()
tb_art_screening_options = {
    'not serious':{
        'Age 0 - 9'   : tb_art_screen_opt[TB_ARTNotServere][TB_ART0_9], 
        'Age 10 - 14' : tb_art_screen_opt[TB_ARTNotServere][TB_ART10_14],
        'Age 15+'     : tb_art_screen_opt[TB_ARTNotServere][TB_ART15p]
    },
    'with serious illness':{
        'Age 0 - 9'   : tb_art_screen_opt[TB_ARTSevereIllness][TB_ART0_9], 
        'Age 10 - 14' : tb_art_screen_opt[TB_ARTSevereIllness][TB_ART10_14],
        'Age 15+'     : tb_art_screen_opt[TB_ARTSevereIllness][TB_ART15p]
    }
}

tb_hrg_opt = TB_screening_alg_hrg_options()
tb_hrg_screening_options ={
    'Prisoners'                                                                         : tb_hrg_opt[TB_HRG_Prisoners],
    'Miners exposed to silica '                                                         : tb_hrg_opt[TB_HRG_Miners],
    'People with risk factors for TB seeking health care'                               : tb_hrg_opt[TB_HRG_SeekingCare],
    'Populations with structural risk factors for TB and limited access to health care' : tb_hrg_opt[TB_HRG_StructuralRFs],
    'General population in settings with high TB'                                       : tb_hrg_opt[TB_HRG_HighTBPrev],
    'People with untreated fibrotic lesions on chest x-ray'                             : tb_hrg_opt[TB_HRG_UntreatedFibrosis],
}

tb_hrg_demographic_indices = {
    'Total pop'      : TG_HRG_Dem_TotalPop, 
    'Proportion 15+' : TG_HRG_Dem_Prop15p,
    'TB infected'    : TG_HRG_Dem_TBInfect,
    'TB diseased'    : TB_HRG_Dem_TBDisease,
    'Relative risk'  : TB_HRG_Dem_RelRisk,
    'PTB'            : TB_HRG_Dem_PTB
}

tb_test_ids = {
     'Clinical assessment: any symptom ' : TB_ClinicalAssessment, 
     'Chest X-ray [ digital (dxr) vs conventional (cxr) ] [ screening test ]' : TB_ChestXRay         , 
     'Computer-aided detection software for automated reading of digital chest radiographs [ screening test ]' : TB_ComputerAidedDtct , 
     'W3SS (3 symptom) single screening algorithm' : TB_W3SS              , 
     '4 symptom screening (no test) / History and physical examination and information on TB' : TB_4SymptomScreening , 
     'C-reactive protein with a cut-off of > 5 mg/L [ screening test ]' : TB_C_ReactiveProtein , 
     'Clinical assessment: any extra-pulmonary TB symptom* ' : TB_ClinicalExtraPulm , 
     'W4SS' : TB_W4SS              , 
     'Single Symptom screening' : TB_SingleSymptomScrn , 
     'Tuberculin Skin Test (TST) ' : TB_TuberculinSkinTest, 
     'TB antigen-based skin tests for the diagnosis of TB infection (TBST)  *, 0% ' : TB_AntigenSkinTest   , 
     'Interferon-Gamma Release Assay (IGRA)*, 100%' : TB_IGRA              , 
     'mWRD single screening algorithm for medical inpatients in settings with TB prevalence > 10%' : TB_mWRDPrevGT10      , 
     'First-line regimen (preferred): TDF + 3TC (or FTC) + DTG' : TB_FirstlinePreferred, 
     'First-line regimen (alternative): TDF + 3TC + EFV 400 mg' : TB_FirstlineAltern   , 
     'Special circumstances: TDF + 3TC (or FTC) + EFV 600mg' : TB_SpecialCircTDF_EFV, 
     'Special circumstances:  AZT + 3TC+ EFV 600mg' : TB_SpecialCircAZT    , 
     'Special circumstances:  TDF + 3TC (or FTC)+ PI' : TB_SpecialCircTDF_PI , 
     'Special circumstances:  TDF + 3TC (or FTC)+  RAL' : TB_SpecialCircTDF_RAL, 
     'Special circumstances:  TAF + 3TC (or FTC)+ DTG' : TB_SpecialCircTAF    , 
     'Special circumstances:  ABC+ 3TC + DTG' : TB_SpecialCircABC    , 
     'First-line regimen (preferred):  ABC + 3TC + DTG' : TB_FirstlineABC_DTG  , 
     'First-line regimen (alternative): ABC + 3TC + LPV/r' : TB_FirstlineABC_LPV  , 
     'First-line regimen (alternative): TAF + 3TC (or FTC) + DTG' : TB_FirstlineTAF_DTG  , 
     'Special circumstances:  ABC+ 3TC + EFV (or NVP)' : TB_SpecialCircABC_EFV, 
     'Special circumstances: ABC + 3TC + RAL' : TB_SpecialCircABC_RAL, 
     'Special circumstances: AZT + 3TC + EFV(or NVP)' : TB_SpecialCircAZT_LPV, 
     'Special circumstances: AZT + 3TC + LPV/r (or RAL)' : TB_SpecialCircAZT_EFV, 
}


from copy import deepcopy
from os import environ
from requests import post, get
from loguru import logger

from AvenirCommon.Util import GBRange
from AvenirCommon.Database import GB_get_db_json

from SpectrumCommon.Const.TB import TB_Fort_Final_Year

from SpectrumCommon.Const.GB.GBVersions import TB_FORT_INPUT_VERSION, TB_FORT_OUTPUT_VERSION

def get_tb_fort_outputs(ISO3, vsn=TB_FORT_OUTPUT_VERSION):
    fort_outputs = GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'tuberculosis', f'fort/outputs/{ISO3}_{vsn}.JSON')

    # Rescaling notification to the reported 2021 values
    # idx   = reported_year-TB_FirstHist_Year
    # scale = reported_notif/fort_outputs['nMid'][idx]
    # fort_outputs['nMid'][idx:] = scale*np.array(fort_outputs['nMid'][idx:])
    # fort_outputs['nLo'][idx:]  = scale*np.array(fort_outputs['nLo'][idx:])
    # fort_outputs['nHi'][idx:]  = scale*np.array(fort_outputs['nHi'][idx:])
    # fort_outputs['nSd'][idx:]  = scale*np.array(fort_outputs['nSd'][idx:])
 
    return fort_outputs


def get_fort_version():
    fort_url = environ.get("AVENIR_SW_FORT_URL", "https://tbbetafort-server.azurewebsites.net")
    response = get(f'{fort_url}/version')
    return response.json()['version']

def call_tb_fort(ISO3, fort_inputs, final_year, tb_slice, fort_slice, HRd = None, HRi = None , ORt = None):
    # fort_inputs = update_tb_fort_inputs(fort_inputs, tb_slice, fort_slice, HRd, HRi, ORt)

    fort_inputs["hRd"][fort_slice] = HRd[tb_slice] if HRd is not None else [1.0]*len(fort_inputs["hRd"][fort_slice])  
    fort_inputs["hRi"][fort_slice] = HRi[tb_slice] if HRi is not None else [1.0]*len(fort_inputs["hRi"][fort_slice])  
    fort_inputs["oRt"][fort_slice] = ORt[tb_slice] if ORt is not None else [1.0]*len(fort_inputs["oRt"][fort_slice])  

    if (ISO3 in ['IDN', 'VNM']):
         fort_inputs['modelType'] = 'failsafe' 
    else: 
         fort_inputs['modelType'] = 'IPn2'


    logger.debug(f"Calling TB Fort API version {get_fort_version()} with model type {fort_inputs['modelType']} for {ISO3} with final year {final_year} and years {fort_inputs['year'][0]}-{fort_inputs['year'][-1]}")

    fort_url = environ.get("AVENIR_SW_FORT_URL", "https://tbbetafort-server.azurewebsites.net")
    # fort_url = "https://tbfort-server.azurewebsites.net"
    # with open('PeteDoddsModel_Inputs.JSON', 'w+') as fp:
        # ujson.dump(fort_inputs, fp)   
    # if type(HRd)!=type(None):
    #     with open('Live_Inputs.JSON', 'r') as fp:
    #         fort_inputs =  ujson.load(fp)   
    response = post(f'{fort_url}/projection', json=fort_inputs)
    # response = post('http://localhost:9004/projection', json=fort_inputs)
    fort_outputs = response.json()


    # with open('PeteDoddsModel_'+ISO3+'_Outputs.JSON', 'w+') as fp:
    #     ujson.dump(fort_outputs, fp)   
    for key in fort_outputs:
        for i,e in enumerate(fort_outputs[key]):
            if e=='NA':
               fort_outputs[key][i] = 0

    # if 'error' in fort_outputs:
        # raise Exception('TB Fort API called failed')
               
    return fort_outputs


def get_tb_fort_inputs(ISO3, final_year, tb_slice, fort_slice, HRd, HRi, ORt,  fort_vsn=TB_FORT_INPUT_VERSION):
    fort_inputs = GB_get_db_json(environ['AVENIR_SW_DEFAULT_DATA_CONNECTION'], 'tuberculosis', f'fort/inputs/{ISO3}_{fort_vsn}.JSON') 
    # fort_inputs = {}
    # null = None
    # fort_inputs["year"] = [2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024,2025,2026,2027,2028,2029,2030]
    # fort_inputs["iHat"] = [282000,282000,280000,275000,270000,264000,259000,254000,250000,245000,239000,233000,225000,218000,206000,197000,187000,177000,168000,160000,151000,143000,null,null,null,null,null,null,null,null,null]
    # fort_inputs["sEi" ] = [165306,160714,153316,144643,135204,126020,118112,111224,104337,97449,90561,65306,76020,69133,60459,38776,39031,33929,29847,26276,25765,26276,null,null,null,null,null,null,null,null,null]
    # fort_inputs["nHat"] = [91101,94957,110289,117600,123127,124262,122198,128844,141157,148936,154694,156539,145323,131677,119592,137960,127407,117705,114233,112597,108714,104854,null,null,null,null,null,null,null,null,null]
    # fort_inputs["sEn" ] = [6972,7267,8440,9000,9423,9510,9352,9861,10803,11398,11839,11980,11122,10077,9152,10558,9751,9008,8742,8617,8320,8025,null,null,null,null,null,null,null,null,null]
    # fort_inputs["mHat"] = [108000,103000,92000,85000,78000,75000,74000,68000,60000,53000,47000,42000,41000,44000,44000,32000,31000,31000,27000,25000,23000,21000,null,null,null,null,null,null,null,null,null]
    # fort_inputs["sEm" ] = [26276,24235,21173,19388,17857,17092,16837,15561,13520,11990,10459,6633,7908,8418,8673,5357,5612,5867,5102,4847,4592,4337,null,null,null,null,null,null,null,null,null]
    # fort_inputs["pHat"] = [null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null]
    # fort_inputs["sEp" ] = [null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null]
    # fort_inputs["TXf" ] = [0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035,0.035]
    # fort_inputs["modelType"] = "IP"
    # #estimate TXf
    #     PropHIVp         = notif_pct[TB_HIVpCases]
    #     PropHIVn         = 1 - PropHIVp
    #     PropHIVponART    = notif_pct[TB_OnARTCases]
    #     PropHIVpNoART    = 1- PropHIVponART
    #     PropHIVponART6m  = 0.1 * PropHIVponART
    #     PropHIVponART12m = 0.9 * PropHIVponART
    #     # check that     PropHIVn+PropHIVpNoART+PropHIVponART6m+PropHIVponART12m=1
    #     TXf              = PropHIVn*(3/100)+PropHIVpNoART*(9/100)+PropHIVponART6m*(6/100)+PropHIVponART12m*(4/100)
    #     # check value for Ethiopia for which TXf ~ 0.035. Check value for Eswatini should closer to the HIV values
    if fort_vsn in ['V1', 'V2']:
        fort_inputs["sEp"] = []
        fort_inputs["tXf"] = []
        fort_inputs["pHat"] = []
        for yr in GBRange(fort_inputs["year"][0], fort_inputs["year"][-1]):
            fort_inputs["sEp"].append(None)
            fort_inputs["pHat"].append(None)
            fort_inputs["tXf"].append(0.035)

        for yr in GBRange(fort_inputs["year"][-1]+1, TB_Fort_Final_Year):
            fort_inputs["year"].append(yr)    
            for key in ["iHat", "sEi", "nHat", "sEn", "mHat", "sEm", "pHat", "sEp"]:
                fort_inputs[key].append(None)
            fort_inputs["tXf"].append(0.035)
        fort_inputs["hRd"] = [1]*len(fort_inputs["year"]) 
        fort_inputs["hRi"] = [1]*len(fort_inputs["year"]) 
        fort_inputs["oRt"] = [1]*len(fort_inputs["year"]) 


    if type(HRd)!=type(None):
        fort_inputs["hRd"][fort_slice] = HRd[tb_slice]  

    if type(HRi)!=type(None):
        fort_inputs["hRi"][fort_slice] = HRi[tb_slice]

    if type(ORt)!=type(None):
        fort_inputs["oRt"][fort_slice] = ORt[tb_slice]
     
    return fort_inputs



import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.HW.HWModVarFields as hwmvf

#######################################################################################################
#                                                                                                     #
#                                      Staff types                                                    #
#                                                                                                     #
#######################################################################################################

def get_HW_num_staff_types(staff_type_records: list):
    return len(staff_type_records)

def get_HW_staff_type_rec_from_curr_ID(staff_type_records: list, curr_ID: int):
    return staff_type_records[curr_ID - 1]

def get_HW_staff_type_rec(staff_type_records: list, ID: str):
    staff_type_recs = [staff_type_rec for staff_type_rec in staff_type_records if staff_type_rec[hwmvf.HW_STAFF_ID] == ID]
    return staff_type_recs[0] if len(staff_type_recs) > 0 else gbc.GB_NOT_FOUND

    # staff_type_rec = None
    
    # st = 0
    # stop = False
    # num_staff_types = get_HW_num_staff_types(staff_type_records)

    # while (st < num_staff_types) and (not stop):

    #     staff_type_ID = get_HW_staff_type_ID(staff_type_records[st])

    #     if staff_type_ID == ID:
    #         staff_type_rec = staff_type_records[st]
    #         stop = True

    #     st += 1

    # return staff_type_rec

def get_HW_staff_type_idx(staff_type_records: list, ID: str):
    indices = [idx for (idx, staff_type_rec) in enumerate(staff_type_records) if staff_type_rec[hwmvf.HW_STAFF_ID] == ID]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND

    # idx = gbc.GB_NOT_FOUND
    
    # st = 0
    # stop = False
    # num_staff_types = get_HW_num_staff_types(staff_type_records)

    # while (st < num_staff_types) and (not stop):

    #     staff_type_ID = get_HW_staff_type_ID(staff_type_records[st])

    #     if staff_type_ID == ID:
    #         idx = st
    #         stop = True

    #     st += 1

    # return idx

###

def get_HW_staff_type_ID(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_ID]

def set_HW_staff_type_ID(staff_type_rec: dict, value: str):
    staff_type_rec[hwmvf.HW_STAFF_ID] = value

###

def get_HW_staff_type_name(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_NAME]

def set_HW_staff_type_name(staff_type_rec: dict, value: str):
    staff_type_rec[hwmvf.HW_STAFF_NAME] = value

###

def get_HW_staff_type_string_constant(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_STR_CONSTANT]

def set_HW_staff_type_string_constant(staff_type_rec: dict, value: str):
    staff_type_rec[hwmvf.HW_STAFF_STR_CONSTANT] = value

###

def get_HW_staff_type_active(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_ACTIVE]

def set_HW_staff_type_active(staff_type_rec: dict, value = bool):
    staff_type_rec[hwmvf.HW_STAFF_ACTIVE] = value

###

def get_HW_staff_type_custom(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_CUSTOM]

def set_HW_staff_type_custom(staff_type_rec: dict, value = bool):
    staff_type_rec[hwmvf.HW_STAFF_CUSTOM] = value

###

def get_HW_staff_type_division(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_DIVISION_MST_ID]

def set_HW_staff_type_division(staff_type_rec: dict, value: int):
    staff_type_rec[hwmvf.HW_STAFF_DIVISION_MST_ID] = value

###

def get_HW_staff_type_static_map(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_STATIC_MAP_MST_ID]

def set_HW_staff_type_default_map(staff_type_rec: dict, value: int):
    staff_type_rec[hwmvf.HW_STAFF_STATIC_MAP_MST_ID] = value

###


def get_HW_staff_type_avg_days_worked_per_year(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_AVG_DAYS_WORKED_PER_YEAR]

def set_HW_staff_type_avg_days_worked_per_year(staff_type_rec: dict, value: int):
    staff_type_rec[hwmvf.HW_STAFF_AVG_DAYS_WORKED_PER_YEAR] = value

###

def get_HW_staff_type_hours_in_a_work_day(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_HOURS_IN_WORK_DAY]

def set_HW_staff_type_hours_in_a_work_day(staff_type_rec: dict, value: int):
    staff_type_rec[hwmvf.HW_STAFF_HOURS_IN_WORK_DAY] = value

###

def get_HW_staff_type_curr_attr_per_year(staff_type_rec: dict, attrit_type_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_CURR_ATTRIT_PER_YEAR][attrit_type_curr_ID - 1]

def set_HW_staff_type_curr_attr_per_year(staff_type_rec: dict, attrit_type_curr_ID: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_CURR_ATTRIT_PER_YEAR][attrit_type_curr_ID - 1] = value

###

def get_HW_staff_type_perc_receiv_attrit_reduc_incents(staff_type_rec: dict, retent_incent_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_PERC_REC_ATTRIT_REDUC_INCENTS][retent_incent_curr_ID - 1]

def set_HW_staff_type_perc_receiv_attrit_reduc_incents(staff_type_rec: dict, retent_incent_curr_ID: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_PERC_REC_ATTRIT_REDUC_INCENTS][retent_incent_curr_ID - 1] = value

###

def get_HW_staff_type_ann_cost_attrit_reduc_incents(staff_type_rec: dict, retent_incent_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_ANNUAL_COST_ATTRIT_REDUC_INCENTS][retent_incent_curr_ID - 1]

def set_HW_staff_type_ann_cost_attrit_reduc_incents(staff_type_rec: dict, retent_incent_curr_ID: int, value = float):
    staff_type_rec[hwmvf.HW_STAFF_ANNUAL_COST_ATTRIT_REDUC_INCENTS][retent_incent_curr_ID - 1] = value

###

def get_HW_staff_type_target_num_per_facility(staff_type_rec: dict, staffing_level: int, facility_type_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_TARGET_STAFF_PER_FACILITY][staffing_level - 1][facility_type_curr_ID - 1]

def set_HW_staff_type_target_num_per_facility(staff_type_rec: dict, staffing_level: int, facility_type_curr_ID: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_TARGET_STAFF_PER_FACILITY][staffing_level - 1][facility_type_curr_ID - 1] = value

###

def get_HW_staff_type_target_retent_incent_levels(staff_type_rec: dict, retent_incent_curr_ID: int, t: int):
    return staff_type_rec[hwmvf.HW_STAFF_TARGET_RETENT_INCENT_LEVELS][retent_incent_curr_ID - 1][t]

def set_HW_staff_type_target_retent_incent_levels(staff_type_rec: dict, retent_incent_curr_ID: int, t: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_TARGET_RETENT_INCENT_LEVELS][retent_incent_curr_ID - 1][t] = value

###

def get_HW_staff_type_num_in_base_year(staff_type_rec: dict, staff_level_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_BASE_YEAR][staff_level_curr_ID - 1]

def set_HW_staff_type_num_in_base_year(staff_type_rec: dict, staff_level_curr_ID: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_BASE_YEAR][staff_level_curr_ID - 1] = value

###

def get_HW_staff_type_target_setting_method(staff_type_rec: dict, staff_level_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_TARGET_SETTING_METHOD][staff_level_curr_ID - 1]

def set_HW_staff_type_target_setting_method(staff_type_rec: dict, staff_level_curr_ID: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_TARGET_SETTING_METHOD][staff_level_curr_ID - 1] = value

###

def get_HW_staff_type_target_staff_pop_ratio(staff_type_rec: dict, staff_level_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_TARGET_STAFF_POP_RATIO][staff_level_curr_ID - 1]

def set_HW_staff_type_target_staff_pop_ratio(staff_type_rec: dict, staff_level_curr_ID: int, value = float):
    staff_type_rec[hwmvf.HW_STAFF_TARGET_STAFF_POP_RATIO][staff_level_curr_ID - 1] = value

###

def get_HW_staff_type_annual_salary(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_ANNUAL_SALARY]

def set_HW_staff_type_annual_salary(staff_type_rec: dict, value = float):
    staff_type_rec[hwmvf.HW_STAFF_ANNUAL_SALARY] = value

###

def get_HW_staff_type_benefits(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_ANNUAL_BENEFITS]

def set_HW_staff_type_benefits(staff_type_rec: dict, value = float):
    staff_type_rec[hwmvf.HW_STAFF_ANNUAL_BENEFITS] = value

###

def get_HW_staff_type_annual_real_salary_incr(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_ANNUAL_REAL_SALARY_INCR]

def set_HW_staff_type_annual_real_salary_incr(staff_type_rec: dict, value = float):
    staff_type_rec[hwmvf.HW_STAFF_ANNUAL_REAL_SALARY_INCR] = value

###

def get_HW_staff_type_annual_benefits_adj(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_ANNUAL_BENEFITS_ADJUST]

def set_HW_staff_type_annual_benefits_adj(staff_type_rec: dict, value: int):
    staff_type_rec[hwmvf.HW_STAFF_ANNUAL_BENEFITS_ADJUST] = value

###

def get_HW_staff_type_scale_up_existing_plans(staff_type_rec: dict, staff_level_curr_ID: int, t: int):
    return staff_type_rec[hwmvf.HW_STAFF_TARGET_SCALE_UP_EXISTING_PLANS][staff_level_curr_ID - 1][t]

def set_HW_staff_type_scale_up_existing_plans(staff_type_rec: dict, staff_level_curr_ID: int, t: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_TARGET_SCALE_UP_EXISTING_PLANS][staff_level_curr_ID - 1][t] = value

###

def get_HW_staff_type_target_graduation(staff_type_rec: dict, t: int):
    return staff_type_rec[hwmvf.HW_STAFF_TARGET_GRADUATION][t]

def set_HW_staff_type_target_graduation(staff_type_rec: dict, t: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_TARGET_GRADUATION][t] = value

###

def get_HW_staff_type_target_enrollment(staff_type_rec: dict, t: int):
    return staff_type_rec[hwmvf.HW_STAFF_TARGET_ENROLLMENT][t]

def set_HW_staff_type_target_enrollment(staff_type_rec: dict, t: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_TARGET_ENROLLMENT][t] = value

###

def get_HW_staff_type_inserv_train_cost_perc_salary(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_INSERV_TRAIN_COST_PERC_SALARY]

def set_HW_staff_type_inserv_train_cost_perc_salary(staff_type_rec: dict, value = float):
    staff_type_rec[hwmvf.HW_STAFF_INSERV_TRAIN_COST_PERC_SALARY] = value

###

def get_HW_inserv_train_type_name(inserv_train_type_rec: dict):
    return inserv_train_type_rec[hwmvf.HW_INSERV_TRAIN_NAME]

def set_HW_inserv_train_type_name(inserv_train_type_rec: dict, value: str):
    inserv_train_type_rec[hwmvf.HW_INSERV_TRAIN_NAME] = value

###

def get_HW_inserv_train_type_inserv_train_cost_per_staff_trained(inserv_train_type_rec: dict):
    return inserv_train_type_rec[hwmvf.HW_INSERV_TRAIN_COST_PER_STAFF_TRAINED]

def set_HW_inserv_train_type_inserv_train_cost_per_staff_trained(inserv_train_type_rec: dict, value = float):
    inserv_train_type_rec[hwmvf.HW_INSERV_TRAIN_COST_PER_STAFF_TRAINED] = value

###

def get_HW_inserv_train_type_inserv_train_num_staff_to_train(inserv_train_type_rec: dict, t: int):
    return inserv_train_type_rec[hwmvf.HW_INSERV_TRAIN_NUM_STAFF_TO_TRAIN][t] 

def set_HW_inserv_train_type_inserv_train_num_staff_to_train(inserv_train_type_rec: dict, t: int, value: int):
    inserv_train_type_rec[hwmvf.HW_INSERV_TRAIN_NUM_STAFF_TO_TRAIN][t]  = value

###

def get_HW_staff_type_preserv_num_schools(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_NUM_SCHOOLS]

def set_HW_staff_type_preserv_num_schools(staff_type_rec: dict, value: int):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_NUM_SCHOOLS] = value

###

def get_HW_staff_type_preserv_annual_cost_per_student(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_ANNUAL_COST_PER_STUDENT]

def set_HW_staff_type_preserv_annual_cost_per_student(staff_type_rec: dict, value = float):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_ANNUAL_COST_PER_STUDENT] = value

###

def get_HW_staff_type_preserv_years_training(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_YEARS_TRAINING]

def set_HW_staff_type_preserv_years_training(staff_type_rec: dict, value: int):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_YEARS_TRAINING] = value

###

def get_HW_staff_type_preserv_curr_enrollment(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_CURR_ENROLLMENT]

def set_HW_staff_type_preserv_curr_enrollment(staff_type_rec: dict, value: int):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_CURR_ENROLLMENT] = value

###

def get_HW_staff_type_preserv_grads_most_recent_year(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_GRADS_MOST_RECENT_YEAR]

def set_HW_staff_type_preserv_grads_most_recent_year(staff_type_rec: dict, value: int):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_GRADS_MOST_RECENT_YEAR] = value

###

def get_HW_staff_type_preserv_inst_name(staff_type_rec: dict, inst_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_INST_NAMES][inst_curr_ID - 1]

def set_HW_staff_type_preserv_inst_name(staff_type_rec: dict, inst_curr_ID: int, value: str):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_INST_NAMES][inst_curr_ID - 1] = value

###

def get_HW_staff_type_preserv_annual_avg_cost_per_student_by_inst(staff_type_rec: dict, inst_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_ANNUAL_AVG_COST_PER_STUDENT_BY_INST][inst_curr_ID - 1]

def set_HW_staff_type_preserv_annual_avg_cost_per_student_by_inst(staff_type_rec: dict, inst_curr_ID: int, value = float):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_ANNUAL_AVG_COST_PER_STUDENT_BY_INST][inst_curr_ID - 1] = value

###

def get_HW_staff_type_avg_years_training_by_inst(staff_type_rec: dict, inst_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_AVG_YEARS_TRAINING_BY_INST][inst_curr_ID - 1]

def set_HW_staff_type_avg_years_training_by_inst(staff_type_rec: dict, inst_curr_ID: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_AVG_YEARS_TRAINING_BY_INST][inst_curr_ID - 1] = value

###

def get_HW_staff_type_preserv_curr_intake_by_inst(staff_type_rec: dict, inst_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_CURR_INTAKE_BY_INST][inst_curr_ID - 1]

def set_HW_staff_type_preserv_curr_intake_by_inst(staff_type_rec: dict, inst_curr_ID: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_CURR_INTAKE_BY_INST][inst_curr_ID - 1] = value

###

def get_HW_staff_type_preserv_grads_most_recent_year_by_inst(staff_type_rec: dict, inst_curr_ID: int):
    return staff_type_rec[hwmvf.HW_STAFF_PRESERV_GRADS_MOST_RECENT_YEAR_BY_INST][inst_curr_ID - 1]

def set_HW_staff_type_preserv_grads_most_recent_year_by_inst(staff_type_rec: dict, inst_curr_ID: int, value: int):
    staff_type_rec[hwmvf.HW_STAFF_PRESERV_GRADS_MOST_RECENT_YEAR_BY_INST][inst_curr_ID - 1] = value

###

def get_HW_staff_perc_time_spent_not_train_serv(staff_type_rec: dict):
    return staff_type_rec[hwmvf.HW_STAFF_PERC_TIME_SPENT_NOT_TRAIN_SERV]

# def set_HW_staff_type_preserv_grads_most_recent_year_by_inst(staff_type_rec: dict, value: int):
#     staff_type_rec[hwmvf.HW_STAFF_PERC_TIME_SPENT_NOT_TRAIN_SERV ] = value

#######################################################################################################
#                                                                                                     #
#                                      Attrition types                                                #
#                                                                                                     #
#######################################################################################################

def get_HW_num_attrit_types(attrit_type_records: list):
    return len(attrit_type_records)

def get_HW_attrit_type_rec(attrit_type_records: list, ID: str):
    attrit_type_recs = [attrit_type_rec for attrit_type_rec in attrit_type_records if attrit_type_rec[hwmvf.HW_ATTRIT_TYPE_ID] == ID]
    return attrit_type_recs[0] if len(attrit_type_recs) > 0 else gbc.GB_NOT_FOUND

    # attrit_type_rec = None
    
    # at = 0
    # stop = False
    # num_attrit_types = get_HW_num_attrit_types(attrit_type_records)

    # while (at < num_attrit_types) and (not stop):

    #     attrit_type_ID = get_HW_attrit_type_ID(attrit_type_records[at])

    #     if attrit_type_ID == ID:
    #         attrit_type_rec = attrit_type_records[at]
    #         stop = True

    #     at += 1

    # return attrit_type_rec

def get_HW_attrit_type_idx(attrit_type_records: list, ID: str):
    indices = [idx for (idx, attrit_type_rec) in enumerate(attrit_type_records) if attrit_type_rec[hwmvf.HW_ATTRIT_TYPE_ID] == ID]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND

    # idx = gbc.GB_NOT_FOUND
    
    # at = 0
    # stop = False
    # num_attrit_types = get_HW_num_attrit_types(attrit_type_records)

    # while (at < num_attrit_types) and (not stop):

    #     attrit_type_ID = get_HW_attrit_type_ID(attrit_type_records[at])

    #     if attrit_type_ID == ID:
    #         idx = at
    #         stop = True

    #     at += 1

    # return idx

###

def get_HW_attrit_type_name(attrit_type_rec: dict):
    return attrit_type_rec[hwmvf.HW_ATTRIT_TYPE_NAME]

def set_HW_attrit_type_name(attrit_type_rec: dict, value: str):
    attrit_type_rec[hwmvf.HW_ATTRIT_TYPE_NAME] = value

###

def get_HW_attrit_type_ID(attrit_type_rec: dict):
    return attrit_type_rec[hwmvf.HW_ATTRIT_TYPE_ID]

def set_HW_attrit_type_ID(attrit_type_rec: dict, value: str):
    attrit_type_rec[hwmvf.HW_ATTRIT_TYPE_ID] = value

#######################################################################################################
#                                                                                                     #
#                                      Retention incentives                                           #
#                                                                                                     #
#######################################################################################################

def get_HW_num_retent_incents(retent_incent_records: list):
    return len(retent_incent_records)

def get_HW_retent_incent_rec_from_curr_ID(retent_incent_records: list, curr_ID: int):
    return retent_incent_records[curr_ID - 1]

def get_HW_retent_incent_rec(retent_incent_records: list, ID: str):
    retent_incent_recs = [retent_incent_rec for retent_incent_rec in retent_incent_records if retent_incent_rec[hwmvf.HW_RETENT_INCENT_ID] == ID]
    return retent_incent_recs[0] if len(retent_incent_recs) > 0 else gbc.GB_NOT_FOUND

    # retent_incent_rec = None
    
    # ri = 0
    # stop = False
    # num_retent_incents = get_HW_num_retent_incents(retent_incent_records)

    # while (ri < num_retent_incents) and (not stop):

    #     retent_incent_ID = get_HW_retent_incent_ID(retent_incent_records[ri])

    #     if retent_incent_ID == ID:
    #         retent_incent_rec = retent_incent_records[ri]
    #         stop = True

    #     ri += 1

    # return retent_incent_rec

def get_HW_retent_incent_idx(retent_incent_records: list, ID: str):
    indices = [idx for (idx, retent_incent_rec) in enumerate(retent_incent_records) if retent_incent_rec[hwmvf.HW_RETENT_INCENT_ID] == ID]
    return indices[0] if len(indices) > 0 else gbc.GB_NOT_FOUND

    # idx = gbc.GB_NOT_FOUND
    
    # ri = 0
    # stop = False
    # num_retent_incents = get_HW_num_retent_incents(retent_incent_records)

    # while (ri < num_retent_incents) and (not stop):

    #     retent_incent_ID = get_HW_retent_incent_ID(retent_incent_records[ri])

    #     if retent_incent_ID == ID:
    #         idx = ri
    #         stop = True

    #     ri += 1

    # return idx

###

def get_HW_retent_incent_name(retent_incent_rec: dict):
    return retent_incent_rec[hwmvf.HW_RETENT_INCENT_NAME]

def set_HW_retent_incent_name(retent_incent_rec: dict, value: str):
    retent_incent_rec[hwmvf.HW_RETENT_INCENT_NAME] = value

###

def get_HW_retent_incent_ID(retent_incent_rec: dict):
    return retent_incent_rec[hwmvf.HW_RETENT_INCENT_ID]

def set_HW_retent_incent_ID(retent_incent_rec: dict, value: str):
    retent_incent_rec[hwmvf.HW_RETENT_INCENT_ID] = value

###

def get_HW_retent_incent_reduc_attrit(retent_incent_rec: dict, attrit_type_curr_ID: int):
    return retent_incent_rec[hwmvf.HW_RETENT_INCENT_PERC_REDUC_ATTRIT][attrit_type_curr_ID - 1]

def set_HW_retent_incent_reduc_attrit(retent_incent_rec: dict, attrit_type_curr_ID: int, value: int):
    retent_incent_rec[hwmvf.HW_RETENT_INCENT_PERC_REDUC_ATTRIT][attrit_type_curr_ID - 1] = value

###

def get_HW_target_scale_up_visited(target_scale_up_visited: list, staff_level_curr_ID: int):
    return target_scale_up_visited[staff_level_curr_ID - 1]

def set_HW_target_scale_up_visited(target_scale_up_visited: list, staff_level_curr_ID: int, value = bool):
    target_scale_up_visited[staff_level_curr_ID - 1] = value



#######################################################################################################
#                                                                                                     #
#                                 Inservice training type (tt) records                                #
#                                                                                                     #
#######################################################################################################

def get_HW_inserv_train_type_staff_type_ID(inserv_train_type_rec: dict):
    return inserv_train_type_rec[hwmvf.HW_INSERV_TRAIN_STAFF_TYPE_TO_TRAIN]

def set_HW_inserv_train_type_staff_type_ID(inserv_train_type_rec: dict, value: str):
    inserv_train_type_rec[hwmvf.HW_INSERV_TRAIN_STAFF_TYPE_TO_TRAIN] = value


#######################################################################################################
#                                                                                                     #
#                                       Intervention DB (IDB)                                         #
#                                                                                                     #
#######################################################################################################


# Keys corresponding to columns. Uncomment/add as needed.

# General - Applies to each module.

IV_ROOT_CONSTANT_KEY_IDB   = 'rootConstant'
IV_MST_ID_KEY_IDB          = 'mstID'
IV_MODULES_KEY_IDB         = 'modMstIDs'
IV_MODULES_ENGLISH_KEY_IDB = 'modCodes'       # just for readability; not used

# For repeated blocks for each module (IHT, TB Costing, LiST Costing; shouldn't need different constants for each)

IV_GROUP_ORDER_KEY_IDB      = 'groupOrder'
IV_GROUP_CURR_ID_KEY_IDB    = 'groupCurrID'    # not needed?
IV_GROUP_MST_ID_KEY_IDB     = 'groupMstID'   
IV_SUBGROUP_CURR_ID_KEY_IDB = 'subgroupCurrID' # not needed?
IV_SUBGROUP_MST_ID_KEY_IDB  = 'subgroupMstID' 
IV_INDENTS_KEY_IDB          = 'indents'         # not needed?
IV_STRING_CONSTANT_KEY_IDB  = 'stringConst'
IV_STRING_NUMBER_KEY_IDB    = 'stringNum'  
IV_ENGLISH_STRING_KEY_IDB   = 'englishString' 

# IHT-specific

IV_INTERV_IMPACT_KEY_IDB       = 'impactInterv'

# '<UH - Community default package>' : '',
# '<UH - Outreach default package>'  : '',
# '<UH - Clinic default package>'    : '',
# '<UH - Hospital default package>'  : '',

IV_AVAIL_COMMUNITY_CHAN_KEY_IDB = 'availCommunityChan'
IV_AVAIL_OUTREACH_CHAN_KEY_IDB  = 'availOutreachChan'
IV_AVAIL_CLINIC_CHAN_KEY_IDB    = 'availClinicChan'
IV_AVAIL_HOSPITAL_CHAN_KEY_IDB  = 'availHospitalChan'

IV_PIN_STRING_CONSTANT_KEY_IDB = 'PINStringConst'
IV_PIN_STRING_NUMBER_KEY_IDB   = 'PINStringNum'
IV_TARG_POP_MST_ID_KEY_IDB     = 'targPopMstID'

IV_PERC_DELIV_KEY_IDB = 'percentDelivered'
# IV_PERC_DELIV_COMMUNITY_CURR_YEAR_KEY_IDB         = 'percDelivCommunityCurrYear'
# IV_PERC_DELIV_COMMUNITY_TARG_YEAR_KE_IDBY         = 'percDelivCommunityTargYear'
# IV_PERC_DELIV_OUTREACH_CURR_YEAR_KEY_IDB          = 'percDelivOutreachCurrYear'
# IV_PERC_DELIV_OUTREACH_TARG_YEAR_KEY_IDB          = 'percDelivOutreachTargYear'
# IV_PERC_DELIV_CLINIC_CURR_YEAR_KEY_IDB            = 'percDelivClinicCurrYear'
# IV_PERC_DELIV_CLINIC_TARG_YEAR_KEY_IDB            = 'percDelivClinicTargYear'
# IV_PERC_DELIV_HOSPITAL_CURR_YEAR_KEY_IDB          = 'percDelivHospitalCurrYear'
# IV_PERC_DELIV_HOSPITAL_TARG_YEAR_KEY_IDB          = 'percDelivHospitalTargYear'
# IV_PERC_DELIV_WASH_CURR_YEAR_KEY_IDB              = 'percDelivWASHCurrYear'
# IV_PERC_DELIV_WASH_TARG_YEAR_KEY_IDB              = 'percDelivWASHTargYear'
# IV_PERC_DELIV_OTHER_NON_HEALTH_CURR_YEAR_KEY_IDB  = 'percDelivOtherNonHealthCurrYear'
# IV_PERC_DELIV_OTHER_NON_HEALTH_TARG_YEAR_KEY_IDB  = 'percDelivOtherNonHealthTargYear'
# IV_PERC_DELIV_PRIVATE_SECTOR_CURR_YEAR_KEY_IDB    = 'percDelivPrivateSectorCurrYear'
# IV_PERC_DELIV_PRIVATE_SECTOR_TARG_YEAR_KEY_IDB    = 'percDelivPrivateSectorTargYear'

# # LiST-specific

IV_NUTRITION_CS_KEY_IDB          = 'isNutritionIV'
IV_BIRTH_OUTCOME_CS_KEY_IDB      = 'isBirthOutcomeIV'
IV_INTERV_TYPE_CS_KEY_IDB        = 'intervType'
IV_QUALITY_MODE_CS_KEY_IDB       = 'QualityMode'
IV_OTHER_MODULE_CS_KEY_IDB       = 'FromOtherModule'
IV_M_CS_KEY_IDB                  = 'MaternalEff_ED'
IV_SB_CS_KEY_IDB                 = 'StillbirthEff_ED'
IV_NN_CS_KEY_IDB                 = 'NNEff_ED'
IV_PNN_CS_KEY_IDB                = 'PNNEff_ED'
IV_ADOL_EFF_CS_KEY_IDB           = 'AdolEff_ED'
IV_COVERAGE_CS_KEY_IDB           = 'Coverage_ED'
IV_DISPLAY_CS_KEY_IDB            = 'Display_ED'
IV_MATERNAL_DISPLAY_CS_KEY_IDB   = 'DisplayMat_ED'
IV_STILLBIRTH_DISPLAY_CS_KEY_IDB = 'DisplaySB_ED'
IV_ADOLESCENT_DISPLAY_CS_KEY_IDB = 'DisplayAdol_ED'
IV_VACCINES_BEDNET_CS_KEY_IDB    = 'VaccinesBednet_ED'
IV_IS_STUNT_INTERV_CS_KEY_IDB    = 'isStuntInterv_ED'

# # TIME Econ-specific

IV_SPECIAL_CASES_IH_KEY_IDB = 'specialCases'

#######################################################################################################
#                                                                                                     #
#                                       Group DB (GDB)                                                #
#                                                                                                     #
#######################################################################################################

# Keys corresponding to columns. Uncomment/add as needed.

# General - Applies to each module.

IV_ROOT_CONSTANT_KEY_GDB    = 'rootConstant'
IV_GROUP_MST_ID_KEY_GDB     = 'groupMstID'
IV_SUBGROUP_MST_ID_KEY_GDB  = 'subgroupMstID'
IV_GROUP_CURR_ID_KEY_GDB    = 'groupCurrID' 
IV_SUBGROUP_CURR_ID_KEY_GDB = 'subgroupCurrID'       
IV_ENGLISH_STRING_KEY_GDB   = 'englishString' 
IV_STRING_CONSTANT_KEY_GDB  = 'stringConst'     
IV_STRING_NUMBER_KEY_GDB    = 'stringNum'  


from SpectrumCommon.Modvars.DP.DPModvarShapes import DP_modvar_shapes
from SpectrumCommon.Modvars.GB.GBDefs import gb_update_years
from SpectrumCommon.Const.DP import *

def DP_UpdateYears(projection, new_projection, params):

    projection[DP_InitialConditionsTag] = new_projection[DP_InitialConditionsTag]

    gb_update_years(projection, params, DP_modvar_shapes)
    pass


PC_VERSION_TAG          = '<PC_Version_V1>'
PC_DATA_CHANGED_TAG     = '<PC_DataChanged_V1>'
PC_PROJ_VALID_TAG       = '<PC_ProjectionValid_V1>'
PC_FILE_NAME_TAG        = '<PC_FileName_V1>'                        
PC_PROJ_VALID_DATE_TAG  = '<PC_ProjectionValidDate_V1>'   
PC_AREA_DATA_TAG        = '<PC_AreaData_V1>'

PC_COST_CAT_RECORDS_TAG                      = '<PC_CostCategoryRecords_V1>'
PC_COST_INPUT_RECORDS_TAG                    = '<PC_CostInputRecords_V3>'
PC_COSTING_SECT_RECORDS_TAG                  = '<PC_CostingSectionRecords_V2>'
PC_ACT_RECORDS_TAG                           = '<PC_ActivityRecords_V2>'
PC_ACT_LINE_ITEM_RECORDS_TAG                 = '<PC_ActivityLineItemRecords_V1>'
PC_ENTER_COSTING_SECT_COSTS_ALL_YEARS_CS_TAG = '<PC_EnterCostingSectionCostsAllYearsCS_V1>'

# Policy and population level interventions
PC_COSTING_SECT_RECORDS_PPLI_TAG                  = '<PC_CostingSectionRecordsPPLI_V1>'
PC_ACT_RECORDS_PPLI_TAG                           = '<PC_ActivityRecordsPPLI_V1>'
PC_ACT_LINE_ITEM_RECORDS_PPLI_TAG                 = '<PC_ActivityLineItemRecordsPPLI_V1>'

# Results

PC_RES_COSTS_BY_SECT_TAG    = '<PC_ResCostsBySection_V1>'
PC_RES_COSTS_BY_SUBSECT_TAG = '<PC_ResCostsBySubsection_V1>'
PC_RES_COSTS_BY_ACT_TAG     = '<PC_ResCostsByActivity_V1>'
PC_RES_COSTS_BY_CAT_TAG     = '<PC_ResCostsByCategory_V1>'

PC_RES_COSTS_BY_SECT_PPLI_TAG    = '<PC_ResCostsBySectionPPLI_V1>'
PC_RES_COSTS_BY_SUBSECT_PPLI_TAG = '<PC_ResCostsBySubsectionPPLI_V1>'
PC_RES_COSTS_BY_ACT_PPLI_TAG     = '<PC_ResCostsByActivityPPLI_V1>'
PC_RES_COSTS_BY_CAT_PPLI_TAG     = '<PC_ResCostsByCategoryPPLI_V1>'

# Meta ModVars. The values inside these do not change. 

PC_STD_ROW_INFO_META_TAG = '<PC_StandardRowInfo_Meta_V1>'
PC_HIV_ROW_INFO_META_TAG = '<PC_HIVRowInfo_Meta_V1>'